package com.futu.course.user.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.futu.course.common.utils.Md5Utils;
import com.futu.course.common.utils.ThreadLocalUtil;
import com.futu.course.common.utils.TokenUtils;
import com.futu.course.minio.service.impl.FileServiceImpl;
import com.futu.course.user.domain.User;
import com.futu.course.user.dto.TokenDto;
import com.futu.course.user.service.UserService;
import com.futu.course.user.mapper.UserMapper;
import io.minio.errors.*;
import org.springframework.stereotype.Service;


import com.futu.course.common.entity.R;
import com.futu.course.common.utils.SnowflakeIdWorker;
import com.futu.course.user.dto.LoginDTO;
import com.futu.course.user.dto.UserDTO;
import com.futu.course.user.dto.UserLoginDto;

import org.apache.commons.codec.binary.Base64;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.springframework.util.DigestUtils;


/**
* @author yuu
* @description 针对表【user(用户表)】的数据库操作Service实现
* @createDate 2025-05-05 18:52:53
*/
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User>
    implements UserService{


    @Autowired
    private UserMapper userMapper;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    private FileServiceImpl fileService;
    public String unload(MultipartFile multipartFile, @RequestParam("uid") Long uid) throws ServerException, InvalidBucketNameException, InsufficientDataException, ErrorResponseException, IOException, NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException {
        User byId = this.getById(uid);
        String s = fileService.uploadImage(multipartFile);
        byId.setAvatar(s);
        boolean b = this.updateById(byId);
        return b?"修改成功":"修改失败";
    }




    // 填写上你的AppID，如何获取AppID自行百度，这步骤很简单
    private final static String APP_ID = "wxdbcbc020c8e4b0d9";
    // 填写上你的AppSecret，如何获取AppSecret自行百度，这步骤很简单
    private final static String APP_SECRET = "60efa9e4ff7fae35c7f561de7763da7a";
    // 微信小程序登录校验请求地址
    private final static String LOGIN_URL = "https://api.weixin.qq.com/sns/jscode2session";
    @Override
    public R appLogin(UserDTO dto) {

        SnowflakeIdWorker worker=new SnowflakeIdWorker(1,1);
        System.out.println(dto);
        String url = LOGIN_URL + "?appid=" + APP_ID + "&secret="+ APP_SECRET + "&grant_type=authorization_code&js_code=" + dto.getCode();
        HttpClient client = HttpClients.createDefault(); // 创建默认http连接
        HttpGet getRequest = new HttpGet(url);// 创建一个post请求
        LoginDTO loginDTO = new LoginDTO();
        try {
            // 用http连接去执行get请求并且获得http响应
            HttpResponse response = client.execute(getRequest);
            // 从response中取到响实体
            HttpEntity entity = response.getEntity();
            // 把响应实体转成文本
            String html = EntityUtils.toString(entity);
            loginDTO = JSON.parseObject(html, LoginDTO.class);
            if(null == loginDTO.getErrCode()) {
                dto.setWxId(loginDTO.getOpenid());
            } else {
                return R.restResult(loginDTO,loginDTO.getErrCode(),loginDTO.getErrMsg());
            }
            List<User> users = userMapper.selectList(new LambdaQueryWrapper<User>().eq(User::getOpenId, dto.getWxId()));
            if(users.size() > 0) {
                User user = users.get(0);
                // 生成一个 256 位（32 字节）的安全密钥
                String jwtToken = TokenUtils.createJwtToken(user.getId().toString());
                user.setToken(jwtToken);
                redisTemplate.opsForValue().set("user",user,1, TimeUnit.DAYS);
                return R.ok(user);
            }
            else {
                User user = new User();
                String sessionKey = loginDTO.getSession_key();

                String encryptedData = dto.getEncryptedData();

                String iv = dto.getIv();

                String decryptedData1 = decryptData(encryptedData, sessionKey, iv);
                System.out.println("解密后的数据：" + decryptedData1);
                String decryptedData = decryptData(encryptedData, sessionKey, iv);
                JSONObject jsonObject = JSON.parseObject(decryptedData);

                String unionId = jsonObject.getString("unionId"); // 提取 unionId

                user.setOpenId(loginDTO.getOpenid());
                user.setSessionKey(loginDTO.getSession_key());
                user.setUnionId(dto.getWxId());
                user.setAvatar(dto.getWxHeadPic());
                user.setUsername(dto.getWxUsername());
                user.setNickname(dto.getWxUsername());
                user.setId(worker.nextId());

//              默认密码是123456
                String salt = Md5Utils.getSalt();
                String encode = Md5Utils.encode("123456" + salt);
                user.setPassword(encode);
                user.setSalt(salt);




//                user.setPassword(null);
                // 生成一个 256 位（32 字节）的安全密钥
                String jwtToken = TokenUtils.createJwtToken(user.getId().toString());
                user.setToken(jwtToken);
                userMapper.insert(user);

                redisTemplate.opsForValue().set("user",user,1, TimeUnit.DAYS);
                return R.ok(user);
            }
        } catch (ClientProtocolException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    @Override
    public R logout(UserDTO dto) {
        if (redisTemplate.opsForValue().get("user") != null)
        {
            redisTemplate.delete("user");
        }
        return R.ok("登出成功");
    }

    @Override
    public R login(UserLoginDto dto) throws InterruptedException {
        RLock lock = redissonClient.getLock(dto.getMobile());
        boolean b = lock.tryLock(3, TimeUnit.SECONDS);
        if (!b)
        {
            return R.failed("请勿重复提交");
        }
        try {
            User user = userMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getMobile, dto.getMobile()));
            if (user == null)
            {
                return R.restResult(null,500,"用户不存在");
            }
            String salt = user.getSalt();
//            输入的加密密码
            String encode = Md5Utils.encode(dto.getPassword() + salt);
            System.err.println(encode);
            //            数据库的加密密码
            String password = user.getPassword();
            System.err.println(password);
            if (encode.equals(password))
            {
                String jwtToken = TokenUtils.createJwtToken(user.getId().toString());
                System.err.println(jwtToken);
//                // 生成一个 256 位（32 字节）的安全密钥
//                SecretKey key = Keys.secretKeyFor(SignatureAlgorithm.HS256);
//                // 使用密钥生成 JWT
//                String jws = Jwts.builder()
//                        .setSubject(user.getId().toString())
//                        .signWith(key)
//                        .compact();
//                user.setToken(jws);
                redisTemplate.opsForValue().set("user",user,1, TimeUnit.DAYS);
                return R.ok(jwtToken);
            }

        }catch (Exception e)
        {
            e.printStackTrace();
            throw e;
        }finally {
            if (lock.isLocked()&&lock.isHeldByCurrentThread())
            {
                lock.unlock();
            }
        }
        return null;
    }

//    @Override
//    public R register(UserRegisterDto dto) {
//        SnowflakeIdWorker worker=new SnowflakeIdWorker(1,1);
//        if (redisTemplate.opsForValue().get("user") != null)
//        {
//            return R.failed("用户已存在");
//        }
//        try {
//            User user = new User();
//            user.setId(worker.nextId());
//        }
//
//        return null;
//    }


    public static String decryptData(String encryptedData, String sessionKey, String iv) throws Exception {
        // Base64 解码
        byte[] encryptedDataBytes = Base64.decodeBase64(encryptedData);
        byte[] sessionKeyBytes = Base64.decodeBase64(sessionKey);
        byte[] ivBytes = Base64.decodeBase64(iv);

        // 初始化 AES 解密器
        Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
        SecretKeySpec keySpec = new SecretKeySpec(sessionKeyBytes, "AES");
        IvParameterSpec ivSpec = new IvParameterSpec(ivBytes);
        cipher.init(Cipher.DECRYPT_MODE, keySpec, ivSpec);

        // 执行解密
        byte[] decryptedBytes = cipher.doFinal(encryptedDataBytes);
        return new String(decryptedBytes, StandardCharsets.UTF_8);
    }







    @Override
    public R getLoginUser(TokenDto token) {

        //解析token
        Long userId = TokenUtils.getUserId(token.getToken());

        System.out.println(userId);
        if (userId != null)
        {
            User user = userMapper.selectById(userId);
            return R.ok(user);
        }
        return null;
    }









}




